# ===================================================================
#'
#' Prepare data for plotting from Raven custom output file
#'
#' This routine takes a custom output filename generated by the Raven modelling software
#' and extracts a single timestamp's worth of data in preparation for plotting using the
#' \code{\link{HRUMap.plot}} function. It also prepares a default legend and plot title
#' from the custom output file name
#'
#' @param custfilename Raven custom output file name (e.g., "SNOW_Monthly_Average_ByHRU.csv")
#' @param date_toplot  date to plot aggregate results for "2007-02-01" (first of month for monthly, first of year for yearly)
#'
#' @return
#'  \item{validHRUIDs}{a vector of HRU ids which are present in the custom output file}
#'  \item{toplot}{a dataframe of data linked to each HRU, where rownames are HRU IDs}
#'  \item{legtitle}{a proposed legend title generated from the custom ouptut filename}
#'  \item{plottitle}{a proposed plot title generated from the custom ouptut filename}
#'
#' @author James R. Craig, University of Waterloo
#'
#' @importFrom rgdal readOGR
#' @importFrom grDevices rgb colorRamp
#'
#' @seealso \code{\link{HRUMap.plot}}; See also the \href{http://raven.uwaterloo.ca/}{Raven web site}
#'
#' @examples
#'
#'\dontrun{
#' HRUdata<-HRUMap.prep.custom("SNOW_Monthly_Average_ByHRU.csv","2007-02-01")
#' HRUMap.plot("HRUs.shp",HRUdata$validHRUIDs,toplot=HRUdata$toplot,plottitle=HRUdata$plotttitle,legtitle=HRUdata$legtitle)
#'}
#' @keywords Raven  Map Plot HRUs
#' @export HRUMap.prep.custom
#'
HRUMap.prep.custom<-function(custfilename, date_toplot)
{
  if (!file.exists(custfilename)){
    print("Warning: the Raven custom ouput file does not exist.")
  }
  HRUdata<-custom.read(custfilename)
  toplot <-(HRUdata[date_toplot,]) # row of Custom data

  if (is.null(toplot)){
    print('Warning: custom plot data is null; invalid plot date?')
  }
  pos<-grep('_ByHRU',custfilename)
  if (length(pos)==0){
    print('Warning: custom plot mau not be a proper Raven custom HRU output.')
  }
  legtitle<-xtsAttributes(HRUdata)$datatype

  cfile<-basename(custfilename)
  plottitle <<-paste(substr(cfile,1,nchar(cfile)-4)," ",date_toplot)


  return (list(validHRUIDs=as.numeric(colnames(HRUdata)),toplot=toplot,legtitle=legtitle,plottitle=plottitle))
}
# ===================================================================
#' HRU plot preparation from table
#'
#' Prepares data for plotting from arbitrary data table
#'
#' @param HRUVec a vector or dataframe column of HRU IDs
#' @param DataVec a vector or dataframe column of corresponding real-valued continuous data to plot
#'
#' @return
#'  \item{validHRUIDs}{a vector of HRU ids which are present in the shapefile}
#'  \item{toplot}{a dataframe of data linked to each HRU, where rownames are HRU IDs}
#'  \item{legtitle}{a proposed legend title}
#'  \item{plottitle}{a proposed plot title generated from the custom ouptut filename}
#'
#' @author James R. Craig, University of Waterloo
#'
#' @seealso \code{\link{HRUMap.plot}}
#' See also the \href{http://raven.uwaterloo.ca/}{Raven web site}
#'
#' @keywords Raven  Map Plot HRUs
#'
#' @export HRUMap.prep.table
#'
HRUMap.prep.table<-function(HRUIDs,DataVec,legtit=NULL,plottit=NULL)
{
  toplot<-data.frame(e=DataVec)
  rownames(toplot)<-HRUIDs

  return (list(validHRUIDs=rownames(toplot),toplot=t(toplot),legtitle=legtit,plottitle=plottit))
}

# ===================================================================
#' Plot continuous data using HRU shapefile
#'
#' Plots Raven custom output or other tabular data to HRU map
#'
#'  @param shpfilename filename of shapefile containing HRU polygons, with one column inidicating Raven HRU ID
#'  @param HRUID_name  name of shapefile column corresponding to HRU ID (default: "ID")
#'  @param validHRUIDs vector of valid modeled HRU IDs
#'  @param toplot  data frame of values to plot with row names equal to HRU IDs
#'  @param plottitle title of plot
#'  @param legtitle legend title for plot
#'  @param dmin data range minimum for determining color scale (optional)
#'  @param dmax data range maximum for determining color scale (optional)
#'
#' @details does not currently support discrete data such as land use
#'
#' @return TRUE if completed successfully
#'
#' @author James R. Craig, University of Waterloo
#'
#' @seealso \code{\link{HRUMap.prep.table}} \code{\link{HRUMap.prep.custom}}
#' See also the \href{http://raven.uwaterloo.ca/}{Raven web site}
#'
#' @examples
#' HRUdata<-HRUMap.prep.custom("SNOW_Monthly_Average_ByHRU.csv","2007-02-01")
#' HRUMap.plot("HRUs.shp",HRUdata$validHRUIDs,toplot=HRUdata$toplot,plottitle=HRUdata$plotttitle,legtitle=HRUdata$legtitle)
#'
#' rvh<-rvh.read("example.rvh")
#' HRUdata<-HRUMap.prep.table(rvh$HRUtable$ID,rvh$HRUtable$Elevation,legtit="Elevation [m]")
#' HRUMap.plot("HRUs.shp",HRUdata$validHRUIDs,toplot=HRUdata$toplot,plottitle=HRUdata$plotttitle,legtitle=HRUdata$legtitle)
#'
HRUMap.plot<-function(shpfilename,HRUID_name='ID',validHRUIDs,toplot,plottitle,legtitle,dmin=1e99,dmax=-1e99,label=FALSE)
{
  if (!file.exists(shpfilename)){
    print("Warning (HRUMap.plot): the shapefile does not exist.")
    return(FALSE)
  }

  #HRUshp<-readShapeSpatial(shpfilename)
  dsn=normalizePath(dirname(shpfilename))
  lay=substr(basename(shpfilename),1,nchar(basename(shpfilename))-4)
  print(lay)
  HRUshp<-readOGR(dsn = dsn,layer = lay)

  #if (!(HRUID_name %within% names(HRUshp@data))){
  #  print("Warning (HRUMap.plot):Invalid HRU ID column identifier")
  #  return(FALSE)
  #}

  HRUIDs<-HRUshp@data[[HRUID_name]] # all HRUIDs in the GIS File
  if (is.null(HRUIDs)){
    print("Warning (HRUMap.plot):Invalid HRU ID column identifier")
    return(FALSE)
  }
  # list of HRU IDs, in same order as in HRU file. Assumes 'ID' is shapefile column which refers to Raven HRU
  #attribute table is HRUshp@data
  #column access through (e.g.) HRUshp@data$Shape_area
  # testing selection routines
  #HRUshp2<-HRUshp[HRUshp@data$SBID==9,]
  #HRUshp3<-HRUshp[HRUshp@data$SBID %in% c(1,5,9),]

  names(HRUshp@data)[names(HRUshp@data) == HRUID_name] <- 'HID' # rename column to HID for access
  invalidHRUs<-HRUshp[!(HRUshp@data$HID %in% validHRUIDs),]
  validHRUs  <-HRUshp[ (HRUshp@data$HID %in% validHRUIDs),]

  if (is.null(validHRUs))
  {
    print("Warning (HRUMap.plot): no valid match between data HRU IDs and shapefile HRU IDs")
    return (FALSE)
  }
  else
  {
    # JRC todo: if data is string-based (e.g., landcover), should plot differently

    # Plot Custom data in valid basins
    HRUIDs<-HRUIDs[HRUIDs %in% validHRUIDs] # valid shape IDs in shapefile

    dd<-coredata(toplot[,HRUIDs]) #data properly sorted

    # generate color scale
    dmin=min(c(min(dd),dmin))
    dmax=max(c(max(dd),dmax))
    drange=((dd-dmin)/(dmax-dmin))
    hc<-rgb(colorRamp(c("azure", "blue"))(drange)/255)

    # plot valid basins using color scale
    plot(validHRUs,col=hc,border=NA)

    # label values
    if (label==TRUE){
      invisible(text(getSpPPolygonsLabptSlots(validHRUs),labels=as.character(round(dd,1)), cex=0.5)) #label with value
      #invisible(text(getSpPPolygonsLabptSlots(validHRUs),labels=as.character(HRUIDs), cex=0.5)) # label with HRU ID
    }

    # Create Legend, title
    title(plottitle)
    cc<-seq(0.0,5,1)/5
    legcolor<-rgb(colorRamp(c("azure", "blue"))(cc)/255)
    nos<-as.character(round(cc*(dmax-dmin)+dmin,1))
    legend(x="bottomright", y="bottomright", legend=nos, legcolor, cex=0.6, bty="n",title=legtitle)
  }

  # Plot invalid basins as gray
  plot(invalidHRUs,add=TRUE,col=rep("gray40",length(invalidHRUs@data)))
}


