#' Read in Raven Hydrograph file
#'
#' hyd.dyGraphs plots modeled vs observed hydrographs when supplied with hydrograph data 
#' structure read using hyd.read() 
#'
#' @param hy hydrograph data structure generated by hyd.read() routine
#' @param timezone data timezone; defaults to UTC
#' @param basins list of subbasin names from hydrograph file. Each subbasin creates separate dygraph plots
#' @param figheight height of figure, in pixels
#'
#' @return a list of plot handles to dygraph plots
#'
#' See also \href{http://raven.uwaterloo.ca/}{Raven page}
#' @keywords Raven hydrograph plotting 
#' @examples
#'
#' # read in Raven Hydrographs file
#' hy<-hyd.read("c:/TEMP/Hydrographs.csv",tz="MST")
#' 
#' # view contents for subbasin 11 as dyGraph
#' res<-hyd.dyGraphs(hy,basins="sub11")
#'
#' @export hyd.dyGraphs
hyd.dyGraphs <- function(hy, timezone="UTC",basins="", figheight=400)
{

  if (basins==""){
    basins<-sub('_obs', "", colnames(hy$hyd))
    basins<-sub('_res', "", basins)
    basins<-unique(basins)
    basins<-basins[-1] # no precip ts
  }
  
  plotfunc <- function(basin){
    hyd <- hyd.extract(subs = basin, hyd = hy)
    flow <- merge(hyd$sim, hyd$obs)
    return(dygraph(flow, group = 'flows', main = basin, height=figheight) %>% 
             dyRangeSelector())
  }
  
  res <- map(basins, plotfunc) # calls plotfunc for each basin in list basins
  return(res)
}
