#' Write/Overwrite Raven .rvh (watershed discretization) file.
#'
#' Given an HRU and SubBasin dataframe, writes to the specified .rvh file or, in the case of rvh.overwrite,
#' just overwrites the :SubBasins-:EndSubBasins and :HRUs-:EndHRUs blocks, retaining all other content.
#'
#' @param filename Name of .rvh file, with extension
#' @param SBtable Valid subbasin dataframe with required columns "SBID", "Name", "Downstream_ID", "Profile",
#'   "ReachLength", and "Gauged". Can have additional columns.
#' @param HRUtable Valid HRU dataframe with required columns 'ID', 'Area', 'Elevation', 'Latitude', 'Longitude',
#'    'SBID', 'LandUse', 'Vegetation', 'SoilProfile', 'Terrain', 'Aquifer', 'Slope', and 'Aspect'. Can have additional columns.
#' @param orig_file original .rvh file, if overwriting. Must be different from  that specified by filename.
#'
#' @details
#'   rvh.write is typically used to create a 'clean' .rvh file. rvh.overwrite is usually used
#'   after reading an original .rvh file and processing the HRU and SubBasin tables, using (e.g., rvh.clean)
#' @return none, generates/revises .rvh file only
#' @author James R. Craig, University of Waterloo
#'
#' @seealso
#' \code{\link{rvh.read}} for the function used to read in the HRU and SubBasin data
#' \code{\link{rvh.cleanHRUs}} for the function used to process HRU dataframes
#' See also the \href{http://raven.uwaterloo.ca/}{Raven web site}
#' @examples
#'   #read in rvh file, clean contents and write back to new file
#'   rvh<-rvh.read("model.rvh")
#'   rvh<-rvh.cleanHRUs(rvh$HRUtable,rvh$SBtable)
#'   rvh.overwrite("cleanmodel.rvh","model.rvh",rvh$HRUtable,rvh$SBtable)
#'   rvh.write("nostrings.rvh",rvh$HRUtable,rvh$SBtable)
#' @keywords Raven  rvh  HRUs write
#' @export rvh.write
rvh.write<-function(filename,SBtable,HRUtable)#add HRUgroups later
{
  write("# -------------------------------------------------",append=FALSE,file=filename)
  write("# .rvh file generated by RavenR rvh.write.R utility",append=TRUE,file=filename)
  write("# -------------------------------------------------",append=TRUE,file=filename)

  write(":SubBasins",append=TRUE,file=filename)
  write(":Properties,SBID,Name,Downstream_ID,Profile,ReachLength,Gauged",append=TRUE,file=filename)
  write(":Units			     ,none,none         ,none		,km				  ,none",append=TRUE,file=filename)
  write.table(SBtable[c('SBID','Name','Downstream_ID','Profile','ReachLength','Gauged')],quote=FALSE,row.names=FALSE,col.names=FALSE,sep=", ",append=TRUE,file=filename)
  write(":EndSubBasins",append=TRUE,file=filename)
  write(" ",append=TRUE,file=filename)

  write(":HRUs",append=TRUE,file=filename)

  write(":Attributes,AREA, ELEVATION,LATITUDE,LONGITUDE,BASIN_ID,LAND_USE_CLASS,VEG_CLASS,SOIL_PROFILE,AQUIFER_PROFILE,TERRAIN_CLASS,SLOPE,ASPECT",append=TRUE,file=filename)
  write(":Units			,km2 , masl     ,deg		 ,deg      ,none		,none          ,none     ,none        ,none           ,none         ,deg  ,degN",append=TRUE,file=filename)

  write.table(HRUtable[c('ID','Area','Elevation','Latitude','Longitude','SBID','LandUse','Vegetation','SoilProfile','Terrain','Aquifer','Slope','Aspect')],quote=FALSE,row.names=FALSE,col.names=FALSE,sep=", ",append=TRUE,file=filename)
  write(":EndHRUs",append=TRUE,file=filename)
  write(" ",append=TRUE,file=filename)
}

#' @describeIn rvh.write Overwrite contents of original .rvh file
#' @export rvh.write
rvh.overwrite<-function(orig_file,filename,SBtable,HRUtable)
{
  write("# -----------------------------------------------------",append=FALSE,file=filename)
  write("# .rvh file modified by RavenR rvh.overwrite.R utility ",append=TRUE,file=filename)
  write("# -----------------------------------------------------",append=TRUE,file=filename)

  lineno<-grep(":SubBasins", readLines(orig_file), value = FALSE)
  lineend<-grep(":EndSubBasins", readLines(orig_file), value = FALSE)
  lines=readLines(orig_file)
  a<-1:length(lines)

  noSB<-lines[!  a %in% (lineno+1):(lineend-1) ] # trim out SB table contents

  lineno<-grep(":HRUs", noSB, value = FALSE)
  lineend<-grep(":EndHRUs", noSB, value = FALSE)
  a<-1:length(noSB)

  noHRUs<-noSB[! a %in% (lineno+1):(lineend-1) ] # trim out HRU table contents


  SBstr<-grep(":SubBasins", noHRUs, value = FALSE)
  SBend<-grep(":EndSubBasins", noHRUs, value = FALSE)
  HRUstr<-grep(":HRUs", noHRUs, value = FALSE)
  HRUend<-grep(":EndHRUs", noHRUs, value = FALSE)

  #insert SB contents
  write(noHRUs[1:SBstr],append=TRUE,file=filename)
  write("  :Properties,SBID,Name,Downstream_ID,Profile,ReachLength,Gauged",append=TRUE,file=filename)
  write("  :Units			     ,none,none         ,none		,km				  ,none",append=TRUE,file=filename)
  write.table(SBtable[c('SBID','Name','Downstream_ID','Profile','ReachLength','Gauged')],quote=FALSE,row.names=FALSE,col.names=FALSE,sep=", ",append=TRUE,file=filename)

  write(noHRUs[SBend:HRUstr],append=TRUE,file=filename)

  write("  :Attributes,AREA, ELEVATION,LATITUDE,LONGITUDE,BASIN_ID,LAND_USE_CLASS,VEG_CLASS,SOIL_PROFILE,AQUIFER_PROFILE,TERRAIN_CLASS,SLOPE,ASPECT",append=TRUE,file=filename)
  write("  :Units			,km2 , masl     ,deg		 ,deg      ,none		,none          ,none     ,none        ,none           ,none         ,deg  ,degN",append=TRUE,file=filename)

  write.table(HRUtable[c('ID','Area','Elevation','Latitude','Longitude','SBID','LandUse','Vegetation','SoilProfile','Terrain','Aquifer','Slope','Aspect')],quote=FALSE,row.names=FALSE,col.names=FALSE,sep=", ",append=TRUE,file=filename)

  write(noHRUs[HRUend:length(noHRUs)],append=TRUE,file=filename)

}
